open! Core
open! Async_kernel
open! Import
open! Auto_mode_alist

let%expect_test "[add]" =
  add
    [ { delete_suffix_and_recur = false
      ; filename_match = "foo" |> Regexp.quote
      ; function_ = Some ("foo-mode" |> Symbol.intern)
      }
    ];
  print_s [%sexp (List.take (Current_buffer.value_exn auto_mode_alist) 1 : t)];
  [%expect
    {|
    ((
      (filename_match          foo)
      (function_               foo-mode)
      (delete_suffix_and_recur false))) |}];
  return ()
;;

let%expect_test "[auto_mode_alist]" =
  print_s [%sexp (Current_buffer.value_exn auto_mode_alist : t)];
  [%expect
    {|
    (((filename_match          foo)
      (function_               foo-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.gpg\\(~\\|\\.~[0-9]+~\\)?\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.elc\\'")
      (function_               elisp-byte-code-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.zst\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.dz\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.xz\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.lzma\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.lz\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.g?z\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.bz2\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.Z\\'")
      (delete_suffix_and_recur true))
     ((filename_match          "\\.vr[hi]?\\'")
      (function_               vera-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\(?:\\.\\(?:rbw?\\|ru\\|rake\\|thor\\|jbuilder\\|rabl\\|gemspec\\|podspec\\)\\|/\\(?:Gem\\|Rake\\|Cap\\|Thor\\|Puppet\\|Berks\\|Vagrant\\|Guard\\|Pod\\)file\\)\\'")
      (function_               ruby-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.re?st\\'")
      (function_               rst-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.py[iw]?\\'")
      (function_               python-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.less\\'")
      (function_               less-css-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.scss\\'")
      (function_               scss-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.awk\\'")
      (function_               awk-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.\\(u?lpc\\|pike\\|pmod\\(\\.in\\)?\\)\\'")
      (function_               pike-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.idl\\'")
      (function_               idl-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.java\\'")
      (function_               java-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.m\\'")
      (function_               objc-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ii\\'")
      (function_               c++-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.i\\'")
      (function_               c-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.lex\\'")
      (function_               c-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.y\\(acc\\)?\\'")
      (function_               c-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.h\\'")
      (function_               c-or-c++-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.c\\'")
      (function_               c-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.\\(CC?\\|HH?\\)\\'")
      (function_               c++-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.[ch]\\(pp\\|xx\\|\\+\\+\\)\\'")
      (function_      c++-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.\\(cc\\|hh\\)\\'")
      (function_               c++-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.\\(bat\\|cmd\\)\\'")
      (function_               bat-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.[sx]?html?\\(\\.[a-zA-Z_]+\\)?\\'")
      (function_               mhtml-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.svgz?\\'")
      (function_               image-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.svgz?\\'")
      (function_               xml-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.x[bp]m\\'")
      (function_               image-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.x[bp]m\\'")
      (function_               c-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.p[bpgn]m\\'")
      (function_               image-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.tiff?\\'")
      (function_               image-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.gif\\'")
      (function_               image-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.png\\'")
      (function_               image-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.jpe?g\\'")
      (function_               image-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.te?xt\\'")
      (function_               text-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.[tT]e[xX]\\'")
      (function_               tex-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ins\\'")
      (function_               tex-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ltx\\'")
      (function_               latex-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.dtx\\'")
      (function_               doctex-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.org\\'")
      (function_               org-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.el\\'")
      (function_               emacs-lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "Project\\.ede\\'")
      (function_               emacs-lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.\\(scm\\|stk\\|ss\\|sch\\)\\'")
      (function_      scheme-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.l\\'")
      (function_               lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.li?sp\\'")
      (function_               lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.[fF]\\'")
      (function_               fortran-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.for\\'")
      (function_               fortran-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.p\\'")
      (function_               pascal-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.pas\\'")
      (function_               pascal-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.\\(dpr\\|DPR\\)\\'")
      (function_               delphi-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ad[abs]\\'")
      (function_               ada-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ad[bs].dg\\'")
      (function_               ada-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.\\([pP]\\([Llm]\\|erl\\|od\\)\\|al\\)\\'")
      (function_               perl-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "Imakefile\\'")
      (function_               makefile-imake-mode)
      (delete_suffix_and_recur false))
     ((filename_match "Makeppfile\\(?:\\.mk\\)?\\'")
      (function_      makefile-makepp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.makepp\\'")
      (function_               makefile-makepp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.mk\\'")
      (function_               makefile-gmake-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.make\\'")
      (function_               makefile-gmake-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "[Mm]akefile\\'")
      (function_               makefile-gmake-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.am\\'")
      (function_               makefile-automake-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.texinfo\\'")
      (function_               texinfo-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.te?xi\\'")
      (function_               texinfo-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.[sS]\\'")
      (function_               asm-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.asm\\'")
      (function_               asm-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.css\\'")
      (function_               css-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.mixal\\'")
      (function_               mixal-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.gcov\\'")
      (function_               compilation-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "/\\.[a-z0-9-]*gdbinit")
      (function_               gdb-script-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "-gdb\\.gdb")
      (function_               gdb-script-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "[cC]hange\\.?[lL]og?\\'")
      (function_               change-log-mode)
      (delete_suffix_and_recur false))
     ((filename_match "[cC]hange[lL]og[-.][0-9]+\\'")
      (function_      change-log-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\$CHANGE_LOG\\$\\.TXT")
      (function_               change-log-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.scm\\.[0-9]*\\'")
      (function_               scheme-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.[ckz]?sh\\'\\|\\.shar\\'\\|/\\.z?profile\\'")
      (function_               sh-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.bash\\'")
      (function_               sh-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\(/\\|\\`\\)\\.\\(bash_\\(profile\\|history\\|log\\(in\\|out\\)\\)\\|z?log\\(in\\|out\\)\\)\\'")
      (function_               sh-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\(/\\|\\`\\)\\.\\(shrc\\|zshrc\\|m?kshrc\\|bashrc\\|t?cshrc\\|esrc\\)\\'")
      (function_               sh-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\(/\\|\\`\\)\\.\\([kz]shenv\\|xinitrc\\|startxrc\\|xsession\\)\\'")
      (function_               sh-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.m?spec\\'")
      (function_               sh-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.m[mes]\\'")
      (function_               nroff-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.man\\'")
      (function_               nroff-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.sty\\'")
      (function_               latex-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.cl[so]\\'")
      (function_               latex-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.bbl\\'")
      (function_               latex-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.bib\\'")
      (function_               bibtex-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.bst\\'")
      (function_               bibtex-style-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.sql\\'")
      (function_               sql-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.m[4c]\\'")
      (function_               m4-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.mf\\'")
      (function_               metafont-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.mp\\'")
      (function_               metapost-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.vhdl?\\'")
      (function_               vhdl-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.article\\'")
      (function_               text-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.letter\\'")
      (function_               text-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.i?tcl\\'")
      (function_               tcl-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.exp\\'")
      (function_               tcl-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.itk\\'")
      (function_               tcl-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.icn\\'")
      (function_               icon-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.sim\\'")
      (function_               simula-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.mss\\'")
      (function_               scribe-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.f9[05]\\'")
      (function_               f90-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.f0[38]\\'")
      (function_               f90-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.indent\\.pro\\'")
      (function_               fundamental-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.\\(pro\\|PRO\\)\\'")
      (function_               idlwave-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.srt\\'")
      (function_               srecode-template-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.prolog\\'")
      (function_               prolog-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.tar\\'")
      (function_               tar-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\.\\(arc\\|zip\\|lzh\\|lha\\|zoo\\|[jew]ar\\|xpi\\|rar\\|cbr\\|7z\\|ARC\\|ZIP\\|LZH\\|LHA\\|ZOO\\|[JEW]AR\\|XPI\\|RAR\\|CBR\\|7Z\\)\\'")
      (function_               archive-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.oxt\\'")
      (function_               archive-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.\\(deb\\|[oi]pk\\)\\'")
      (function_               archive-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\`/tmp/Re")
      (function_               text-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "/Message[0-9]*\\'")
      (function_               text-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\`/tmp/fol/")
      (function_               text-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.oak\\'")
      (function_               scheme-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.sgml?\\'")
      (function_               sgml-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.x[ms]l\\'")
      (function_               xml-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.dbk\\'")
      (function_               xml-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.dtd\\'")
      (function_               sgml-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ds\\(ss\\)?l\\'")
      (function_               dsssl-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.jsm?\\'")
      (function_               javascript-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.json\\'")
      (function_               javascript-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.jsx\\'")
      (function_               js-jsx-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.[ds]?vh?\\'")
      (function_               verilog-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.by\\'")
      (function_               bovine-grammar-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.wy\\'")
      (function_               wisent-grammar-mode)
      (delete_suffix_and_recur false))
     ((filename_match "[:/\\]\\..*\\(emacs\\|gnus\\|viper\\)\\'")
      (function_               emacs-lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\`\\..*emacs\\'")
      (function_               emacs-lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "[:/]_emacs\\'")
      (function_               emacs-lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "/crontab\\.X*[0-9]+\\'")
      (function_               shell-script-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ml\\'")
      (function_               lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ld[si]?\\'")
      (function_               ld-script-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "ld\\.?script\\'")
      (function_               ld-script-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.xs\\'")
      (function_               c-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.x[abdsru]?[cnw]?\\'")
      (function_               ld-script-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.zone\\'")
      (function_               dns-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.soa\\'")
      (function_               dns-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.asd\\'")
      (function_               lisp-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.\\(asn\\|mib\\|smi\\)\\'")
      (function_      snmp-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.\\(as\\|mi\\|sm\\)2\\'")
      (function_               snmpv2-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.\\(diffs?\\|patch\\|rej\\)\\'")
      (function_      diff-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.\\(dif\\|pat\\)\\'")
      (function_               diff-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.[eE]?[pP][sS]\\'")
      (function_               ps-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\.\\(?:PDF\\|DVI\\|OD[FGPST]\\|DOCX?\\|XLSX?\\|PPTX?\\|pdf\\|djvu\\|dvi\\|od[fgpst]\\|docx?\\|xlsx?\\|pptx?\\)\\'")
      (function_               doc-view-mode-maybe)
      (delete_suffix_and_recur false))
     ((filename_match "configure\\.\\(ac\\|in\\)\\'")
      (function_      autoconf-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.s\\(v\\|iv\\|ieve\\)\\'")
      (function_      sieve-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "BROWSE\\'")
      (function_               ebrowse-tree-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ebrowse\\'")
      (function_               ebrowse-tree-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "#\\*mail\\*")
      (function_               mail-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.g\\'")
      (function_               antlr-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.mod\\'")
      (function_               m2-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ses\\'")
      (function_               ses-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.docbook\\'")
      (function_               sgml-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.com\\'")
      (function_               dcl-mode)
      (delete_suffix_and_recur false))
     ((filename_match "/config\\.\\(?:bat\\|log\\)\\'")
      (function_      fundamental-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\.\\(?:[iI][nN][iI]\\|[lL][sS][tT]\\|[rR][eE][gG]\\|[sS][yY][sS]\\)\\'")
      (function_               conf-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.la\\'")
      (function_               conf-unix-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.ppd\\'")
      (function_               conf-ppd-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "java.+\\.conf\\'")
      (function_               conf-javaprop-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.properties\\(?:\\.[a-zA-Z0-9._-]+\\)?\\'")
      (function_               conf-javaprop-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.toml\\'")
      (function_               conf-toml-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.desktop\\'")
      (function_               conf-desktop-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\`/etc/\\(?:DIR_COLORS\\|ethers\\|.?fstab\\|.*hosts\\|lesskey\\|login\\.?de\\(?:fs\\|vperm\\)\\|magic\\|mtab\\|pam\\.d/.*\\|permissions\\(?:\\.d/.+\\)?\\|protocols\\|rpc\\|services\\)\\'")
      (function_               conf-space-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "\\`/etc/\\(?:acpid?/.+\\|aliases\\(?:\\.d/.+\\)?\\|default/.+\\|group-?\\|hosts\\..+\\|inittab\\|ksysguarddrc\\|opera6rc\\|passwd-?\\|shadow-?\\|sysconfig/.+\\)\\'")
      (function_               conf-mode)
      (delete_suffix_and_recur false))
     ((filename_match "[cC]hange[lL]og[-.][-0-9a-z]+\\'")
      (function_      change-log-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "/\\.?\\(?:gitconfig\\|gnokiirc\\|hgrc\\|kde.*rc\\|mime\\.types\\|wgetrc\\)\\'")
      (function_               conf-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "/\\.\\(?:enigma\\|gltron\\|gtk\\|hxplayer\\|net\\|neverball\\|qt/.+\\|realplayer\\|scummvm\\|sversion\\|sylpheed/.+\\|xmp\\)rc\\'")
      (function_               conf-mode)
      (delete_suffix_and_recur false))
     ((filename_match
       "/\\.\\(?:gdbtkinit\\|grip\\|orbital/.+txt\\|rhosts\\|tuxracer/options\\)\\'")
      (function_               conf-mode)
      (delete_suffix_and_recur false))
     ((filename_match "/\\.?X\\(?:default\\|resource\\|re\\)s\\>")
      (function_               conf-xdefaults-mode)
      (delete_suffix_and_recur false))
     ((filename_match "/X11.+app-defaults/\\|\\.ad\\'")
      (function_      conf-xdefaults-mode)
      (delete_suffix_and_recur false))
     ((filename_match "/X11.+locale/.+/Compose\\'")
      (function_      conf-colon-mode)
      (delete_suffix_and_recur false))
     ((filename_match "/X11.+locale/compose\\.dir\\'")
      (function_      conf-javaprop-mode)
      (delete_suffix_and_recur false))
     ((filename_match "\\.~?[0-9]+\\.[0-9][-.0-9]*~?\\'")
      (delete_suffix_and_recur true))
     ((filename_match "\\.\\(?:orig\\|in\\|[bB][aA][kK]\\)\\'")
      (delete_suffix_and_recur true))
     ((filename_match
       "[/.]c\\(?:on\\)?f\\(?:i?g\\)?\\(?:\\.[a-zA-Z0-9._-]+\\)?\\'")
      (function_               conf-mode-maybe)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.[1-9]\\'")
      (function_               nroff-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.tgz\\'")
      (function_               tar-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.tbz2?\\'")
      (function_               tar-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.txz\\'")
      (function_               tar-mode)
      (delete_suffix_and_recur false))
     ((filename_match          "\\.tzst\\'")
      (function_               tar-mode)
      (delete_suffix_and_recur false))) |}];
  return ()
;;
